@@ -242,13 +242,13 @@ module Cc1 #:nodoc:
       binding.memory.read
     end
 
-    def assign!(val)
+    def assign!(val, src = nil, br = nil)
       # NOTE: Length of the incomplete array type should be deducted while
       #       initializer evaluation.  So, adjustment of the assigning value
       #       can be done at this point by Value#coerce_to(type).
       # NOTE: Domain of the assigning value must be narrowed before writing to
       #       the memory by Value#coerce_to(type).
-      binding.memory.write(val.coerce_to(type))
+      binding.memory.write(val.coerce_to(type), src, br)
     end
 
     def uninitialize!
@@ -257,13 +257,23 @@ module Cc1 #:nodoc:
 
     def narrow_value_domain!(op, val)
       assign!(type.arbitrary_value) unless self.value
+
       self.value.narrow_domain!(op, val.coerce_to(type))
+      # NOTE: Write via memory to correctly propagate inner variable's
+      #       mutation to its outer variable.
+      binding.memory._cascade_update(nil, nil)
+
       self.value.exist?
     end
 
     def widen_value_domain!(op, val)
       assign!(type.nil_value) unless self.value
+
       self.value.widen_domain!(op, val.coerce_to(type))
+      # NOTE: Write via memory to correctly propagate inner variable's
+      #       mutation to its outer variable.
+      binding.memory._cascade_update(nil, nil)
+
       self.value.exist?
     end
 
@@ -328,60 +338,93 @@ module Cc1 #:nodoc:
       super(mem, dcl_or_def, type, scope)
       # TODO: If too slow, make an index of inner variables.
       @inner_variables = create_inner_variables(type.unqualify, scope)
+      @representative_element = create_representative_element(type.unqualify)
+    end
+
+    attr_reader :representative_element
+
+    def assign!(val, src = nil, br = nil)
+      super
+      if @representative_element
+        if val.undefined?
+          repr_val = @representative_element.type.undefined_value
+        else
+          repr_val = @representative_element.type.arbitrary_value
+        end
+        @representative_element.assign!(repr_val, src, br)
+      end
     end
 
     def enter_value_versioning_group
-      super
       if @inner_variables
         @inner_variables.each do |inner|
           inner.enter_value_versioning_group
         end
       end
+      super
+      if @representative_element
+        @representative_element.enter_value_versioning_group
+      end
     end
 
     def leave_value_versioning_group(raise_complement)
-      super
       if @inner_variables
         @inner_variables.each do |inner|
           inner.leave_value_versioning_group(raise_complement)
         end
       end
+      super
+      if @representative_element
+        @representative_element.leave_value_versioning_group(raise_complement)
+      end
     end
 
     def begin_value_versioning
-      super
       if @inner_variables
         @inner_variables.each do |inner|
           inner.begin_value_versioning
         end
       end
+      super
+      if @representative_element
+        @representative_element.begin_value_versioning
+      end
     end
 
     def end_value_versioning
-      super
       if @inner_variables
         @inner_variables.each do |inner|
           inner.end_value_versioning
         end
       end
+      super
+      if @representative_element
+        @representative_element.end_value_versioning
+      end
     end
 
-    def rollback_latest_value_version!
-      super
+    def thin_latest_value_version!(with_rollback)
       if @inner_variables
         @inner_variables.each do |inner|
-          inner.rollback_latest_value_version!
+          inner.thin_latest_value_version!(with_rollback)
         end
       end
+      super
+      if @representative_element
+        @representative_element.thin_latest_value_version!(with_rollback)
+      end
     end
 
     def rollback_all_value_versions!
-      super
       if @inner_variables
         @inner_variables.each do |inner|
           inner.rollback_all_value_versions!
         end
       end
+      super
+      if @representative_element
+        @representative_element.rollback_all_value_versions!
+      end
     end
 
     def inner?
@@ -439,6 +482,17 @@ module Cc1 #:nodoc:
         CompositeMemberVariable.new(win, self, memb.type, memb.name)
       end
     end
+
+    def create_representative_element(type)
+      if type.array?
+        mem = binding.memory.create_unmapped_window
+        ArrayElementVariable.new(mem, self, type.base_type, 0).tap do |var|
+          var.assign!(type.base_type.undefined_value)
+        end
+      else
+        nil
+      end
+    end
   end
 
   module Nameable
@@ -481,6 +535,7 @@ module Cc1 #:nodoc:
     end
 
     Summary = Struct.new(:object_id, :name, :type, :memory)
+    private_constant :Summary
   end
 
   class TemporaryVariable < OuterVariable
@@ -505,6 +560,7 @@ module Cc1 #:nodoc:
     end
 
     Summary = Struct.new(:object_id, :type, :memory)
+    private_constant :Summary
   end
 
   class InnerVariable < OuterVariable
@@ -632,21 +688,21 @@ module Cc1 #:nodoc:
       rollback_all_global_variables_value! if current_scope.global?
     end
 
-    def declare(dcl)
+    def declare(dcl, br)
       if var = lookup(dcl.identifier.value)
         var.declarations_and_definitions.push(dcl)
         return var
       end
 
       # NOTE: External variable may have undefined values.
-      define_variable(dcl, dcl.type, allocate_memory(dcl),
+      define_variable(dcl, br, dcl.type, allocate_memory(dcl),
                       dcl.type.undefined_value)
     end
 
-    def define(dcl_or_def, init_val = nil)
-      if storage_duration_of(dcl_or_def) == :static
-        # NOTE: Value of the static duration object should be arbitrary because
-        #       execution of its accessors are out of order.
+    def define(dcl_or_def, br, init_val = nil)
+      if storage_duration_of(dcl_or_def) == :static && !dcl_or_def.type.const?
+        # NOTE: Value of the inconstant static duration variable should be
+        #       arbitrary because execution of its accessors are out of order.
         #       So, a value of the initializer should be ignored.
         init_val = dcl_or_def.type.arbitrary_value
       else
@@ -656,14 +712,17 @@ module Cc1 #:nodoc:
       if var = lookup(dcl_or_def.identifier.value)
         if var.scope == current_scope
           var.declarations_and_definitions.push(dcl_or_def)
-          var.assign!(init_val)
+          var.value.force_overwrite!(init_val.coerce_to(var.type))
+          if repr_elem = var.representative_element
+            repr_elem.value.force_overwrite!(repr_elem.type.arbitrary_value)
+          end
           return var
         end
       end
 
       # NOTE: Domain of the init-value will be restricted by type's min-max in
       #       define_variable.
-      define_variable(dcl_or_def, dcl_or_def.type,
+      define_variable(dcl_or_def, br, dcl_or_def.type,
                       allocate_memory(dcl_or_def), init_val)
     end
 
@@ -672,7 +731,7 @@ module Cc1 #:nodoc:
 
       # NOTE: Domain of the init-value will be restricted by type's min-max in
       #       define_variable.
-      define_variable(nil, type, mem, init_val)
+      define_variable(nil, nil, type, mem, init_val)
     end
 
     def lookup(name_str)
@@ -777,9 +836,9 @@ module Cc1 #:nodoc:
     end
 
     private
-    def define_variable(dcl_or_def, type, mem, init_val)
+    def define_variable(dcl_or_def, br, type, mem, init_val)
       var = create_variable(dcl_or_def, type, mem)
-      var.assign!(init_val)
+      var.assign!(init_val, dcl_or_def, br)
 
       if var.named?
         @named_variables.last[var.name] = var
@@ -813,6 +872,8 @@ module Cc1 #:nodoc:
 
     def rollback_all_global_variables_value!
       @named_variables.first.each_value do |var|
+        # NOTE: Rollback effects recorded to global variables because execution
+        #       of its accessors are out of order.
         var.rollback_all_value_versions!
       end
     end
@@ -870,8 +931,8 @@ module Cc1 #:nodoc:
       if type.return_type.function?
         interp.create_tmpvar
       else
-        retn_type = type.return_type
-        interp.create_tmpvar(retn_type, retn_type.return_value)
+        ret_type = type.return_type
+        interp.create_tmpvar(ret_type, ret_type.return_value)
       end
     end
 
@@ -935,7 +996,8 @@ module Cc1 #:nodoc:
           sink = arg
         end
 
-        sink.assign!(sink.type.return_value)
+        ret_val = sink.type.return_value
+        sink.assign!(ret_val, funcall_expr, interp.current_branch)
         interp.notify_variable_value_updated(expr, sink)
 
         # NOTE: Returning a value via a pointer parameter can be considered as
@@ -1114,13 +1176,19 @@ module Cc1 #:nodoc:
       @value
     end
 
-    def write(val)
+    def write(val, src, br)
       if @value
-        @value.overwrite!(val)
+        @value.overwrite!(val, TransitionTag.new([src], [br]))
       else
-        @value = VersionedValue.new(val)
+        @value = VersionedValue.new(val, TransitionTag.new([src], [br]))
       end
     end
+
+    def _cascade_update(src, br)
+      # NOTE: This method will be called only from # #narrow_value_domain! and
+      #       #widen_value_domain! of Variable to propagate memory mutation to
+      #       the upper MemoryBlock from MemoryWindow.
+    end
   end
 
   class MemoryBlock < Memory
@@ -1132,44 +1200,54 @@ module Cc1 #:nodoc:
     attr_reader :windows
 
     def create_window(offset, byte_size)
-      win = MemoryWindow.new(self, @address + offset, byte_size)
-      win.on_written += method(:handle_written_through_window)
-      @windows.push(win)
-      win
+      MemoryWindow.new(self, @address + offset, byte_size).tap do |win|
+        win.on_written += method(:handle_written_through_window)
+        @windows.push(win)
+      end
     end
 
-    def write(val)
+    def create_unmapped_window
+      UnmappedMemoryWindow.new(self).tap do |win|
+        win.on_written += method(:handle_written_through_window)
+      end
+    end
+
+    alias :_orig_write :write
+    def write(val, src, br)
       super
       if !@windows.empty? and
-          @value.array? && val.array? or
-          @value.composite? && val.composite?
-        single_val = val.to_single_value
-        @windows.zip(single_val.values).each do |win, inner_val|
-          win.write(inner_val, false)
+          @value.array? && val.array? or @value.composite? && val.composite?
+        @windows.zip(val.to_single_value.values).each do |win, inner_val|
+          win.write(inner_val, src, br, false)
         end
       end
     end
 
     protected
     def create_value_from_windows
-      case
-      when @value.scalar?
-        @value
-      when @value.array?
-        ArrayValue.new(@windows.map { |w| w.create_value_from_windows })
-      when @value.composite?
-        CompositeValue.new(@windows.map { |w| w.create_value_from_windows })
+      if @value
+        case
+        when @value.scalar?
+          @value
+        when @value.array?
+          ArrayValue.new(@windows.map { |w| w.create_value_from_windows })
+        when @value.composite?
+          CompositeValue.new(@windows.map { |w| w.create_value_from_windows })
+        else
+          __NOTREACHED__
+        end
+      else
+        nil
       end
     end
 
     private
-    def handle_written_through_window(*)
-      val = create_value_from_windows
-
-      if @value
-        @value.overwrite!(val)
-      else
-        @value = VersionedValue.new(val)
+    def handle_written_through_window(win, src, br)
+      if val = create_value_from_windows
+        unless win.read.test_must_be_undefined.true?
+          val = val.to_defined_value
+        end
+        _orig_write(val, src, br)
       end
     end
   end
@@ -1192,15 +1270,25 @@ module Cc1 #:nodoc:
       @owner.dynamic?
     end
 
-    def write(val, cascade = true)
-      super(val)
-      on_written.invoke(self) if cascade
+    def write(val, src, br, cascade = true)
+      super(val, src, br)
+      _cascade_update(src, br) if cascade
+    end
+
+    def _cascade_update(src, br)
+      on_written.invoke(self, src, br)
     end
 
     private
-    def handle_written_through_window(*)
+    def handle_written_through_window(win, src, br)
       super
-      on_written.invoke(self)
+      _cascade_update(src, br)
+    end
+  end
+
+  class UnmappedMemoryWindow < MemoryWindow
+    def initialize(owner)
+      super(owner, owner.address, owner.byte_size)
     end
   end
 
