@@ -240,12 +240,12 @@ module CBuiltin #:nodoc:
       @cur_fun = nil
     end
 
-    def check(retn_stmt)
+    def check(ret_stmt)
       return unless @cur_fun.explicitly_typed?
 
-      if retn_type = @cur_fun.type.return_type
-        if !retn_type.void? && retn_stmt.expression.nil?
-          W(retn_stmt.location, @cur_fun.identifier.value)
+      if ret_type = @cur_fun.type.return_type
+        if !ret_type.void? && ret_stmt.expression.nil?
+          W(ret_stmt.location, @cur_fun.identifier.value)
         end
       end
     end
@@ -259,11 +259,11 @@ module CBuiltin #:nodoc:
     mark_as_unique
 
     private
-    def check(retn_stmt)
+    def check(ret_stmt)
       return unless @cur_fun.implicitly_typed?
 
-      if retn_stmt.expression.nil?
-        W(retn_stmt.location, @cur_fun.identifier.value)
+      if ret_stmt.expression.nil?
+        W(ret_stmt.location, @cur_fun.identifier.value)
       end
     end
   end
@@ -294,9 +294,9 @@ module CBuiltin #:nodoc:
       @cur_fun = nil
     end
 
-    def check_explicit_return(retn_stmt, *)
+    def check_explicit_return(ret_stmt, *)
       if @cur_fun
-        if @cur_fun.implicitly_typed? && retn_stmt.expression.nil?
+        if @cur_fun.implicitly_typed? && ret_stmt.expression.nil?
           W(@cur_fun.location, @cur_fun.identifier.value)
         end
       end
@@ -469,11 +469,11 @@ module CBuiltin #:nodoc:
     mark_as_unique
 
     private
-    def check(retn_stmt)
-      if retn_stmt.expression
-        if retn_type = @cur_fun.type.return_type
-          if retn_type.void? && (retn_type.const? || retn_type.volatile?)
-            W(retn_stmt.location, @cur_fun.identifier.value)
+    def check(ret_stmt)
+      if ret_stmt.expression
+        if ret_type = @cur_fun.type.return_type
+          if ret_type.void? && (ret_type.const? || ret_type.volatile?)
+            W(ret_stmt.location, @cur_fun.identifier.value)
           end
         end
       end
@@ -771,8 +771,8 @@ module CBuiltin #:nodoc:
 
     private
     def check(dcl_or_def)
-      if retn_type = dcl_or_def.type.return_type
-        if retn_type.const? || retn_type.volatile?
+      if ret_type = dcl_or_def.type.return_type
+        if ret_type.const? || ret_type.volatile?
           W(dcl_or_def.location)
         end
       end
@@ -782,6 +782,8 @@ module CBuiltin #:nodoc:
   class W1066 < PassiveCodeCheck
     def_registrant_phase Cc1::Prepare2Phase
 
+    include Cc1::InterpreterMediator
+
     # NOTE: All messages of cc1-phase code check should be unique till function
     #       step-in analysis is supported.
     mark_as_unique
@@ -797,13 +799,13 @@ module CBuiltin #:nodoc:
     end
 
     private
-    def check(*, org_var, res_var)
-      if @rvalues && org_var.type.floating?
-        case expr = @rvalues[org_var]
-        when Cc1::AdditiveExpression, Cc1::MultiplicativeExpression
-          if org_var.type.same_as?(from_type) && res_var.type.same_as?(to_type)
-            W(expr.location)
-          end
+    def check(*, orig_var, rslt_var)
+      return unless @rvalues && orig_var.type.floating?
+
+      case expr = @rvalues[orig_var]
+      when Cc1::AdditiveExpression, Cc1::MultiplicativeExpression
+        if orig_var.type.same_as?(from_type) && rslt_var.type.same_as?(to_type)
+          W(expr.location)
         end
       end
     end
@@ -812,13 +814,13 @@ module CBuiltin #:nodoc:
       @rvalues = {}
     end
 
-    def handle_additive(expr, *, res_var)
-      memorize_rvalue_derivation(res_var, expr)
+    def handle_additive(expr, *, rslt_var)
+      memorize_rvalue_derivation(rslt_var, expr)
     end
 
-    def handle_multiplicative(expr, *, res_var)
+    def handle_multiplicative(expr, *, rslt_var)
       unless expr.operator.type == "%"
-        memorize_rvalue_derivation(res_var, expr)
+        memorize_rvalue_derivation(rslt_var, expr)
       end
     end
 
@@ -827,11 +829,15 @@ module CBuiltin #:nodoc:
     end
 
     def from_type
-      @interp.float_t
+      float_t
     end
 
     def to_type
-      @interp.double_t
+      double_t
+    end
+
+    def interpreter
+      @interp
     end
   end
 
@@ -844,11 +850,11 @@ module CBuiltin #:nodoc:
 
     private
     def from_type
-      @interp.float_t
+      float_t
     end
 
     def to_type
-      @interp.long_double_t
+      long_double_t
     end
   end
 
@@ -861,11 +867,11 @@ module CBuiltin #:nodoc:
 
     private
     def from_type
-      @interp.double_t
+      double_t
     end
 
     def to_type
-      @interp.long_double_t
+      long_double_t
     end
   end
 
